from __future__ import annotations

import argparse
import dataclasses
import typing

import ldaptool.decode.arguments
from ldaptool._utils import argclasses


def _parser_add_attributes(parser: argparse.ArgumentParser, dest: str) -> None:
    parser.add_argument(
        metavar="attributes",
        dest=dest,
        nargs="*",
        help="""
            Attributes to lookup (and columns to display in tables).
            Fake attributes `dndomain`, `dnpath` an `dnfullpath` are available (created from dn).
        """,
    )


@dataclasses.dataclass(slots=True, kw_only=True)
class Arguments(ldaptool.decode.arguments.Arguments):
    # overwrite fields for fake attributes to remove them from argparse;
    # we enable those based on the attribute list
    dndomain: bool = False
    dnpath: bool = False
    dnfullpath: bool = False

    attributes: list[str] = dataclasses.field(default_factory=list, metadata=argclasses.manual(_parser_add_attributes))
    columns: list[str] = dataclasses.field(default_factory=list)
    filter: typing.Optional[str] = dataclasses.field(default=None, metadata=argclasses.arg(help="LDAP query filter"))
    find: typing.Optional[str] = dataclasses.field(
        default=None,
        metadata=argclasses.arg(help="Account/Name/Email to search for (builds filter around it)"),
    )
    # TODO: not calling ldapsearch anymore...
    # debug: bool = dataclasses.field(
    #     default=False,
    #     metadata=argclasses.arg("-d", help="Show arguments to ldapsearch"),
    # )
    gc: bool = dataclasses.field(
        default=False,
        metadata=argclasses.arg(help="Query global catalogue (and forest root as search base)"),
    )
    realm: str = dataclasses.field(metadata=argclasses.arg(required=True, help="Realm to search in"))
    server: typing.Optional[str] = dataclasses.field(
        default=None,
        metadata=argclasses.arg(
            help="""
                Server of realm to connect to
                (attributes like lastLogon are not replicated and can vary between servers)
            """,
        ),
    )
    all: bool = dataclasses.field(
        default=False,
        metadata=argclasses.arg(
            help="Get all results (pagination) instead of only first 1000",
        ),
    )
    krb: bool = dataclasses.field(
        default=False,
        metadata=argclasses.arg(
            help="Use kerberos authentication (ticket must be already present)",
        ),
    )
    base: typing.Optional[str] = dataclasses.field(
        default=None,
        metadata=argclasses.arg(
            "-b",
            help="Explicit search base (defaults to root of domain / forest with --gc)",
        ),
    )

    def __post_init__(self) -> None:
        if not self.filter is None:
            if not self.find is None:
                raise SystemExit("Can't use both --find and --filter")
        elif not self.find is None:
            find = self.find
            self.filter = (
                f"(|(sAMAccountName={find})(email={find})(mail={find})(proxyAddresses=smtp:{find})(description={find}))"
            )
        else:
            # probably doesn't like empty filter?
            self.filter = "(objectClass=*)"

        # extract special attribute names
        self.columns = self.attributes  # use all names for columns (headings and their order)
        attributes_set: dict[str, str] = {arg.lower(): arg for arg in self.attributes}  # index by lowercase name
        # create fake attributes on demand
        if attributes_set.pop("dndomain", ""):
            self.dndomain = True
        if attributes_set.pop("dnpath", ""):
            self.dnpath = True
        if attributes_set.pop("dnfullpath", ""):
            self.dnfullpath = True
        # store remaining attributes (with original case)
        self.attributes = list(attributes_set.values())
        if self.columns and not self.attributes:
            # if we only wanted fake attributes, make sure we only request 'dn' - empty list would query all attributes
            self.attributes = ["dn"]
